import { createApp } from 'vue'
import { createPinia } from 'pinia'

import '../node_modules/bulma/css/bulma.css'
import './assets/main.css'

import App from './App.vue'
import router from './router'
import i18n from '@/i18n'

import { SocketIoClientService } from '@/services/SocketIoClientService'
import { LoggingService } from '@/services/LoggingService'
import { AuthenticationService } from './services/AuthenticationService'
import { GameService } from './services/GameService'
import { PersistenceService } from './services/PersistenceService'

const app = createApp(App)

app.use(createPinia())
app.use(i18n)
app.use(router)

app.provide('socket', new SocketIoClientService(import.meta.env.VITE_SOCKET_URL))
app.provide('logger', new LoggingService())
app.provide('auth', new AuthenticationService())
app.provide('game', new GameService())
PersistenceService.getInstance()

app.mount('#app')
