import { appConfigDir, appLocalDataDir, appCacheDir, appDataDir, join } from '@tauri-apps/api/path'
import { writeTextFile, readTextFile, exists, createDir } from '@tauri-apps/api/fs'
import type { StorageInterface } from './StorageInterface'
import { ServiceBase } from './ServiceBase'

export class TauriFileStorageService extends ServiceBase implements StorageInterface {
  constructor() {
    super()
    this.showDirs()
  }

  async showDirs() {
    this.logger.debug(`=> appConfigDir ${await appConfigDir()}`)
    this.logger.debug(`=> appLocalDataDir ${await appLocalDataDir()}`)
    this.logger.debug(`=> appCacheDir ${await appCacheDir()}`)
    this.logger.debug(`=> appDataDir ${await appDataDir()}`)
  }

  async saveUserDataText(fileName: string, content: any) {
    const userAppDir = await appDataDir()
    await this.ensureDirExists(userAppDir)
    const filePath = await join(userAppDir, fileName + '.txt')
    await this.write(filePath, content, false)
  }

  async saveUserDataJson(fileName: string, content: any) {
    const userAppDir = await appDataDir()
    await this.ensureDirExists(userAppDir)
    const filePath = await join(userAppDir, fileName + '.json')
    await this.write(filePath, content, true)
  }

  async readUserDataText(fileName: string) {
    const userAppDir = await appDataDir()
    await this.ensureDirExists(userAppDir)
    const filePath = await join(userAppDir, fileName + '.txt')
    const content = await this.read(filePath, false)
    return content
  }

  async readUserDataJson(fileName: string) {
    const userAppDir = await appDataDir()
    await this.ensureDirExists(userAppDir)
    const filePath = await join(userAppDir, fileName + '.json')
    const content = await this.read(filePath, true)
    return content
  }

  async saveConfigData(content: any) {
    const userAppDir = await appConfigDir()
    await this.ensureDirExists(userAppDir)
    const filePath = await join(userAppDir, 'config.json')
    await this.write(filePath, content)
  }

  async readConfigData() {
    const userAppDir = await appConfigDir()
    await this.ensureDirExists(userAppDir)
    const filePath = await join(userAppDir, 'config.json')
    const content = await this.read(filePath, true)
    return content
  }

  private async ensureDirExists(dir: string) {
    const dirExists = await exists(dir)
    if (!dirExists) {
      await createDir(dir)
    }
  }

  private async write(filePath: string, content: any, json: boolean = false) {
    this.logger.trace(`write ${filePath}`, content)
    if (json) {
      return writeTextFile(filePath, JSON.stringify(content, null, 2))
    }
    return writeTextFile(filePath, content)
  }

  private async read(filePath: string, json: boolean = false) {
    let content = null
    try {
      if (await exists(filePath)) {
        content = await readTextFile(filePath)

        if (json) {
          content = JSON.parse(content as string)
        }
      }
    } catch (error) {
      this.logger.error(error)
    }
    return content
  }
}
